import os

from toga_dummy import test_implementation

globals().update(
    test_implementation.create_impl_tests(
        os.path.abspath(
            os.path.join(
                os.path.dirname(os.path.dirname(__file__)),
                'toga_django')
        )
    )
)



from .base import Widget
from .. import impl


class Box(Widget):
    def create(self):
        self._impl = impl.Box(
            id=self.id,
            style=self.style,
        )

    def add_child(self, child):
        self._impl.add_child(child._impl)





class Label:
    def __init__(self, id, text, alignment=None, ports=None, on_press=None, style=None):
        self.id = id
        self._impl = None

        self._text = text
        self.style = style

    def __html__(self):
        return '<span id="toga:%s" class="toga Label" style="%s" data-toga-class="toga.Label" data-toga-parent="%s" data-toga-ports="%s">%s</span>' % (
            self.id,
            self.style,
            self.parent.id,
            '',  #  self.ports,
            self.text,
        )

    @property
    def text(self):
        return self._text

    @text.setter
    def text(self, value):
        if self._impl:
            self._impl.innerText = value




class Widget:
    def __init__(self, interface):
        self.interface = interface
        self.interface._impl = self
        self._container = None
        self.create()

    def handler(self, fn, name):
        if hasattr(fn, '__self__'):
            ref = '(%s,%s-%s)' % (fn.__self__.id, self.id, name)
        else:
            ref = '%s-%s' % (self.id, name)

        return ref

    @property
    def ports(self):
        return ",".join(
            "%s=%s" % (name, widget.id)
            for name, widget in self.__dict__.items()
            if isinstance(widget, Widget)
        )

    def set_app(self, app):
        pass

    def set_window(self, window):
        pass

    @property
    def container(self):
        return self._container

    @container.setter
    def container(self, container):
        self._container = container
        if self.constraints and self.native:
            self._container.native.addSubview_(self.native)
            self.constraints.container = container

        for child in self.interface.children:
            child._impl.container = container
        self.interface.rehint()

    def set_enabled(self, value):
        self.native.set_sensitive(value)

    ### APPLICATOR

    def set_bounds(self, x, y, width, height):
        # No implementation required here; the new sizing will be picked up
        # by the box's allocation handler.
        pass

    def set_alignment(self, alignment):
        pass

    def set_hidden(self, hidden):
        pass

    def set_font(self, font):
        pass

    def set_color(self, color):
        pass

    def set_background_color(self, color):
        pass


    ### INTERFACE

    def add_child(self, child):
        pass
        # if self._container:
            # child._set_container(self._container)

    def rehint(self):
        pass

    def set_font(self, font):
        self.interface.factory.not_implemented('Widget.set_font()')



from django.core.urlresolvers import reverse, resolve
from django.utils.html import escape

from .base import Widget
from ..libs import List as TogaList, SimpleListElement as TogaSimpleListElement


class SimpleListElement(Widget):
    def __init__(self, content, detail=None, **style):
        super(SimpleListElement, self).__init__(**style)

        self.content = content
        self.detail = detail

        self.startup()

    def startup(self):
        pass

    def materialize(self):
        return TogaSimpleListElement(
            widget_id=self.widget_id,
            content=escape(self.content),
            delete_url=reverse(self.detail, kwargs={'pk': self.content.id})
        )

    def _set_window(self, window):
        super()._set_window(window)
        if self.on_press:
            self.window.callbacks[(self.widget_id, 'on_press')] = self.on_press


class List(Widget):
    IMPL_CLASS = TogaList

    def __init__(self, source=None, detail=None, item_class=None, on_item_press=None, **style):
        super(List, self).__init__(**style)
        self.source = source
        self.detail = detail
        self.item_class = item_class
        self.on_item_press = on_item_press

        self.children = []

        self.startup()

    def startup(self):
        pass

    def materialize(self):
        children = []
        if self.source:
            api_view = resolve(reverse(self.source)).func
            for child in api_view.view_class().get_queryset():
                children.append(self.item_class(child, self.detail).materialize())
        else:
            for child in self.children:
                children.add(child.materialize())

        return TogaList(
            widget_id=self.widget_id,
            children=children,
            create_url=reverse(self.source),
            on_item_press=self.handler(self.on_item_press, 'on_item_press') if self.on_item_press else None
        )

    def add(self, content):
        if self.source:
            raise Exception("Can't manually add to an API-sourced list")
        self.children.append(self.item_class(content, self.detail))

    def _set_app(self, app):
        for child in self.children:
            child.app = app

    def _set_window(self, window):
        for child in self.children:
            child.window = window

        if self.on_item_press:
            self.window.callbacks[(self.widget_id, 'on_item_press')] = self.on_item_press

    # def _hint_size(self, width, height, min_width=None, min_height=None):
    #     if width is not None:
    #         self.width = width
    #     else:
    #         del(self.width)

    #     if min_width is not None:
    #         self.min_width = min_width
    #     else:
    #         del(self.min_width)

    #     if height is not None:
    #         self.height = height
    #     else:
    #         del(self.height)

    #     if min_height is not None:
    #         self.min_height = min_height
    #     else:
    #         del(self.min_height)

    # def _update_child_layout(self, **style):
    #     """Force a layout update on children of this container.

    #     The update request can be accompanied by additional style information
    #     (probably min_width, min_height, width or height) to control the
    #     layout.
    #     """
    #     for child in self.children:
    #         if child.is_container:
    #             child._update_layout()

    # def _set_frame(self, frame):
    #     print("SET FRAME", self, frame.origin.x, frame.origin.y, frame.size.width, frame.size.height)
    #     self._impl.setFrame_(frame)
    #     self._impl.setNeedsDisplay_(True)
    #     for child in self.children:
    #         layout = child.layout
    #         child._set_frame(NSRect(NSPoint(layout.left, layout.top), NSSize(layout.width, layout.height)))




class WebView:
    def __init__(self, id=None, url=None, on_key_down=None, style=None):
        self.id = id
        self._impl = None
        self._url = url
        self.style = style

    def __html__(self):
        return """
            <iframe id="toga:%s" class="toga WebView" style="%s" src="%s" data-toga-class="toga.WebView" data-toga-ports="%s">
            </iframe>""" % (
                self.id,
                self.style,
                self.url if self.url else '',
                '',  #  self.ports,
            )

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, value):
        self._url = value
        if self._impl:
            self._impl.src = value

    def set_content(self, root_url, content):
        pass

    # def evaluate(self, javascript):
    #     raise NotImplementedError('Webview widget must define evaluate()')




class Container:
    def __init__(self):
        self._content = None

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, widget):
        self._content = widget
        self._content._container = self

    @property
    def root_content(self):
        return self._content

    @root_content.setter
    def root_content(self, widget):
        self._content = widget
        self._content._container = self
        # Make the constraints object a root container.
        self._constraints.make_root()

    def update_layout(self, **style):
        pass



import importlib
import os

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class App(AppConfig):
    name = 'toga'
    label = 'toga'
    verbose_name = _("Toga")
    path = os.path.dirname(__file__)


def TogaApp(module_name):
    module = importlib.import_module(module_name + ".app")
    app = module.main()
    app._module = module
    return app




from . import django
from .app import App, MainWindow
# from .color import color
# from .command import Command
# from .font import font
from .widgets.box import *
from .widgets.button import *
# from .widgets.canvas import Canvas
# from .widgets.detailedlist import DetailedList
# from .widgets.icon import *
# from .widgets.image import *
# from .widgets.imageview import *
from .widgets.label import *
# from .widgets.multilinetextinput import *
# from .widgets.numberinput import NumberInput
# from .widgets.optioncontainer import *
# from .widgets.passwordinput import *
# from .widgets.progressbar import *
# from .widgets.scrollcontainer import *
# from .widgets.selection import Selection
# from .widgets.slider import *
# from .widgets.splitcontainer import *
# from .widgets.switch import *
# from .widgets.table import *
from .widgets.textinput import *
# from .widgets.tree import *
from .widgets.webview import *
from .window import Window


__all__ = [
    '__version__',
    'django',
    'App', 'MainWindow',
    # 'color',
    # 'Command',
    # 'font',
    'Box',
    'Button',
    # 'Canvas',
    # 'DetailedList',
    # 'Icon',
    # 'Image',
    # 'ImageView',
    'Label',
    # 'MultilineTextInput',
    # 'NumberInput',
    # 'OptionContainer',
    # 'PasswordInput',
    # 'ProgressBar',
    # 'ScrollContainer',
    # 'Selection',
    # 'Slider',
    # 'SplitContainer',
    # 'Switch',
    # 'Table',
    'TextInput',
    # 'Tree',
    'WebView',
    'Window',
]

# Examples of valid version strings
# __version__ = '1.2.3.dev1'  # Development release 1
# __version__ = '1.2.3a1'     # Alpha Release 1
# __version__ = '1.2.3b1'     # Beta Release 1
# __version__ = '1.2.3rc1'    # RC Release 1
# __version__ = '1.2.3'       # Final Release
# __version__ = '1.2.3.post1' # Post Release 1

__version__ = '0.3.0.dev5'



#/usr/bin/env python
import io
import re
import sys

from setuptools import setup, find_packages

if sys.version_info[:3] < (3, 4):
    raise SystemExit("Toga requires Python 3.4+.")


with io.open('toga_django/__init__.py', encoding='utf8') as version_file:
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file.read(), re.M)
    if version_match:
        version = version_match.group(1)
    else:
        raise RuntimeError("Unable to find version string.")


with io.open('README.rst', encoding='utf8') as readme:
    long_description = readme.read()


setup(
    name='toga-django',
    version=version,
    description='A Django backend for the Toga widget toolkit.',
    long_description=long_description,
    author='Russell Keith-Magee',
    author_email='russell@keith-magee.com',
    url='http://pybee.org/toga',
    packages=find_packages(exclude='tests'),
    include_package_data=True,
    install_requires=[
        'django==1.10.6',
        'django-environ==0.4.1',
        'toga-core==%s' % version,
    ],
    tests_require=[
        'toga-dummy==%s' % version
    ],
    license='New BSD',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Environment :: MacOS X :: Cocoa',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Software Development',
        'Topic :: Software Development :: User Interfaces',
        'Topic :: Software Development :: Widget Sets',
    ],
    test_suite='tests',
    zip_safe=False,
)




class SimpleListElement:
    def __init__(self, id, content, delete_url=None, on_press=None, style=None):
        self.id = id

        self.content = content
        self.delete_url = delete_url
        self.on_press = on_press
        self.style = style

    def __html__(self):
        return '<tr id="toga:%s" data-toga-class="toga.SimpleListElement" data-toga-parent="%s" data-toga-ports="%s" data-toga-delete-url="%s" data-toga-on-press="%s"><td>%s</td><td style="width:1em;">%s</td></tr>' % (
            self.id,
            self.parent.id,
            '',  #  self.ports,
            self.delete_url if self.delete_url else '',
            self.parent.on_item_press if self.parent.on_item_press else '',
            self.content,
            '<i class="fa fa-times"></i>' if self.delete_url else ''
        )

    def text(self, value):
        self.impl.innerHTML = '<td>%s</td>' % value

    def remove(self):
        self.impl.parentNode.removeChild(self.impl)



class List:
    def __init__(self, id, children, create_url, on_item_press=None):
        self.id = id
        self.create_url = create_url
        self.on_item_press = on_item_press

        self.children = children
        for child in self.children:
            child.parent = self

    def __html__(self):
        lines = [
            '<table id="%s" data-toga-class="toga.List" data-toga-parent="%s" data-toga-ports="%s" data-toga-create-url="%s" data-toga-on-item-press="%s" class="table table-striped">' % (
                self.id,
                self.parent.id,
                '',  # ",".join(
                #     "%s=%s" % (name, id)
                #     for name, id in self.ports.items()
                # ),
                self.create_url,
                self.on_item_press
            )
        ]
        lines.append('<tbody>')
        for child in self.children:
            lines.append(child.__html__())
        lines.append('</tbody>')
        lines.append('</table>')
        return '\n'.join(lines)

    def add(self, content):
        id = next_id()
        child = SimpleListElement(id, content, on_press=self.on_item_press)

        child.parent = self
        self.impl.children[0].innerHTML += child.__html__()

        child.impl = dom.document.getElementById(id)

        fn = dom.window.toga.handler(self.on_item_press, child)
        child.impl.addEventListener('click', fn)

        return child

    def add_waiting(self):
        self.add('<i class="fa fa-spinner fa-spin"></i>')





class Box:
    def __init__(self, id, style=None):
        self.id = id
        self._impl = None

        self.style = style
        self.children = []

    def add_child(self, child):
        self.children.append(child)
        child.parent = self

    def __html__(self):
        lines = [
            '<div id="toga:%s" class="toga Box" style="%s" data-toga-class="toga.Box" data-toga-parent="%s" data-toga-ports="%s">' % (
                self.id,
                self.style,
                self.parent.id,
                '',  #  self.ports,
            )
        ]
        for child in self.children:
            lines.append(child.__html__())
        lines.append('</div>')
        return '\n'.join(lines)





class TextInput:
    def __init__(self, id, initial, placeholder, readonly, style=None):
        self.id = id
        self._impl = None
        self.initial = initial
        self.placeholder = placeholder
        self.readonly = readonly
        self.style = style

    def __html__(self):
        return '<input id="toga:%s" class="toga TextInput" style="%s" data-toga-class="toga.TextInput" data-toga-parent="%s" data-toga-ports="%s" type="text" value="%s"%s%s>' % (
            self.id,
            self.style,
            self.parent.id,
            '',  #  self.ports,
            self.initial,
            ' placeholder="%s"' % self.placeholder if self.placeholder else '',
            ' disabled' if self.readonly else '',
        )

    @property
    def value(self):
        if self._impl:
            return self._impl.value
        return self.initial

    @value.setter
    def value(self, value):
        if self._impl:
            self._impl.value = value

    def clear(self):
        self._impl.value = ''



# try:
#     import dom
# except ImportError:
#     # The dom module doesn't actually exist; it's a proxy for the browser DOM.
#     # However, if we don't try to import it.
#     pass

from .app import App
from .box import Box
from .button import Button
from .label import Label
from .textinput import TextInput
from .webview import WebView
from .window import Window


# _counter = 1000


# def next_widget_id():
#     global _counter
#     _counter += 1
#     return 'NEW%s' % _counter



def bootstrap_App(element):
    app = App(element.dataset.togaName, element.dataset.togaAppId, element.id[5:])

    element.toga = app
    app.impl = element


def bootstrap_Window(element):
    title = dom.document.title
    content = dom.document.querySelector('[data-toga-parent="' + element.id[5:] + '"]')

    window = Window(element.id[5:], title, content=content)
    element.toga = window
    window.impl = element


def bootstrap_Box(element):
    widget = Box(element.id[5:])

    element.toga = widget
    widget._impl = element


def bootstrap_Label(element):
    widget = Label(element.id[5:], element.innerHTML)

    element.toga = widget
    widget._impl = element


def bootstrap_TextInput(element):
    initial = element.getAttribute('value')
    placeholder = element.getAttribute('placeholder')
    readonly = element.getAttribute('disabled') is not None

    widget = TextInput(element.id[5:], initial, placeholder, readonly)

    element.toga = widget
    widget._impl = element


# def bootstrap_SimpleListElement(element):
#     widget = SimpleListElement(element.id[5:], element.innerHTML, element.dataset.togaDeleteUrl)

#     fn = dom.window.toga.handler(element.dataset.togaOnPress, widget)
#     element.addEventListener('click', fn)

#     element.toga = widget
#     widget._impl = element


# def bootstrap_List(element):
#     children = element.querySelectorAll('[data-toga-parent="' + element.id + '"]')

#     widget = List(element.id[5:], children, element.dataset.togaCreateUrl, element.dataset.togaOnItemPress)

#     element.toga = widget
#     widget._impl = element

def bootstrap_Button(element):
    widget = Button(element.id[5:], element.innerHTML)
    fn = dom.window.toga.handler(element.dataset.togaOnPress, widget)
    element.addEventListener('click', fn)

    element.toga = widget
    widget._impl = element


def bootstrap_WebView(element):
    widget = WebView(element.id[5:])

    element.toga = widget
    widget._impl = element
